package mailbeans;

import java.io.*;
import java.net.*;
import java.util.*;
import java.beans.*;
import java.awt.event.*;
import java.awt.*;
/**
 * This class implementes the Post Office Protocol - Version 3. <br>
 * This protocol is specified in the RFC 1081
 * 
 */
public class POP3 extends MailBeansComponent {
	public String username="";
	public String password="";
	public String servername="";
	public String lable="POP3";
	public boolean invisible = false;
	public boolean mimemessage = false;
	public boolean deleteMessages = false;
	
	private Transport trans;
	private java.io.DataInputStream inStream;
	private java.io.DataOutputStream outStream;
	private java.util.Vector msgList;
	protected transient PropertyChangeSupport propertyChange;
/**
 * Constructor
 */
public POP3() {
	super();
}
/**
 * The addPropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
	getPropertyChange().addPropertyChangeListener(listener);
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String createFilename() {
	return null;
}
/**
 * Deletes all messages from the POP3 server. This method isn't used anymore.
 * @exception IOException
 */
private void deleteAllMessages() throws IOException{
	output("delete all message ");
	String[] msgListItem;// = (String[])msgList.elementAt(nr);
	for (Enumeration e = msgList.elements() ; e.hasMoreElements() ;) {
		msgListItem = (String[])e.nextElement();
		String response = trans.doCommand("DELE "+msgListItem[0]);
		if (isErrorResponse(response)) {
			throw new IOException(response);
		}
	}
}
/**
 * Delete the in nr specified Message.
 * @return int
 * @param nr int
 * @exception java.io.IOException The exception description.
 */
private int deleteMessage(int nr) throws IOException {
	output("delete message "+nr);
	String[] msgListItem = (String[])msgList.elementAt(nr);
	String response = trans.doCommand("DELE "+msgListItem[0]);
	if (isErrorResponse(response)) {
		throw new IOException(response);
	}
	
	return msgList.size();
}
/**
 * The firePropertyChange method was generated to support the propertyChange field.
 */
public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
}
/**
 * get-method for the deleteMessages property.
 * @return boolean
 */
public boolean getDeleteMessages() {
	return deleteMessages;
}
/**
 * This method was created in VisualAge.
 * @return boolean
 */
public boolean getInvisible() {
	return invisible;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getLable() {
	return lable;
}
/**
 * Receives the message with the number nr from the server
 * @return Message
 * @param nr int
 * @exception java.io.IOException The exception description.
 */
private Message getMessage(int nr) throws IOException {
	//System.out.println("get message "+nr);
	output("get message "+(nr+1)+"/"+msgList.size());
	String[] msgListItem = (String[])msgList.elementAt(nr);
	String response = trans.doCommand("RETR "+msgListItem[0]);
	if (isErrorResponse(response)) {
		throw new IOException(response);
	}
	if (mimemessage) {
		return new MimeMessage(trans.receive("."));
	} else {
		return new Message(trans.receive("."));
	}
}
/**
 * Get list of messages which are available on the server.
 * @return int
 */
private int getMessageList() throws IOException {
	String response = trans.doCommand("LIST");
	if (isErrorResponse(response)) {
		throw new IOException("connection lost");
	}
	String[] list = trans.receive(".");
	for (int i=0; i<list.length; i++) {
		StringTokenizer st = new StringTokenizer(list[i]);
		String[] msgListItem = new String[2];
		int n = 0;
		while (st.hasMoreTokens()) {
			if (n<2) msgListItem[n++]=st.nextToken();
			else break;
		}
		msgList.addElement(msgListItem);
		//System.out.println(msgListItem[0]+ " & "+msgListItem[1]);
	}
	output(msgList.size()+" Messages found");
	return msgList.size();
}
/**
 * This method was created in VisualAge.
 * @return boolean
 */
public boolean getMimeMessage() {
	return mimemessage;
}
/**
 * This method was created in VisualAge.
 * @return java.awt.Dimension
 */
public Dimension getMinimumSize() {
	return new Dimension(50,50);
}
/**
 * get-method for the password property.
 * @return java.lang.String
 */
public String getPassword() {
	return password;
}
/**
 * This method was created in VisualAge.
 * @return java.awt.Dimension
 */
public Dimension getPreferredSize() {
	return new Dimension(100,50);
}
/**
 * Accessor for the propertyChange field.
 */
protected PropertyChangeSupport getPropertyChange() {
	if (propertyChange == null) {
		propertyChange = new java.beans.PropertyChangeSupport(this);
	};
	return propertyChange;
}
/**
 * get-method for the servername property.
 * @return java.lang.String
 */
public String getServername() {
	return servername;
}
/**
 * get-method for the deleteMessages property.
 * @return java.lang.String
 */
public String getUsername() {
	return username;
}
/**
 * Initalize
 */
private void init() {
	msgList=null;
	msgList=new Vector();
}
/**
 * This method was created by a SmartGuide.
 * @param input java.lang.String
 */
private boolean isErrorResponse (String response) {
	return !response.startsWith("+OK");
}
/**
 * Logon on server
 * @exception IOException
 */
private void logon ( ) throws IOException {
	output("logon");
	//System.out.println("Send USER");
	String response = trans.doCommand("USER "+username);
	//System.out.println(response);
	if (isErrorResponse(response)) {
		throw new IOException(response);
	}

	//System.out.println("Send PASS "+password);
	response =  trans.doCommand("PASS "+password);
	//System.out.println(response);
	if (isErrorResponse(response)) {
		output("Error in logon");
		throw new IOException(response);
	}
	
}
/**
 * This method was created in VisualAge.
 */
private void message(Message msg) {
	MessageEvent e = new MessageEvent(this, msg);
	for (int i=0; i<messageListener.size(); i++) {
		MessageListener ml = (MessageListener) messageListener.elementAt(i);
		ml.handleMessage(e);
	}
}
/**
 * This method was created in VisualAge.
 */
private void open() throws IOException {
	init();
	trans = new Transport(servername, 110);
	if (trans!=null) {
		//System.out.println("Open connection");
		output("Connect to "+servername);
		trans.open();
	}
}
/**
 * This method was created in VisualAge.
 */
public void output(String text) {
	OutputEvent e = new OutputEvent(this, text);
	for (int i=0; i<outputListener.size(); i++) {
		OutputListener ml = (OutputListener) outputListener.elementAt(i);
		ml.handleOutput(e);
	}
}
/**
 * This method was created in VisualAge.
 * @param g java.awt.Graphics
 */
public void paint(Graphics g) {
	if (!invisible) {
		int width = getSize().width;
		int height = getSize().height;
		g.setColor(getBackground());
		g.fillRect(0,0, width-1, height-1);
		g.setColor(getForeground());
		g.setFont(getFont());
		FontMetrics fm = g.getFontMetrics();
		g.drawString(lable , (width-fm.stringWidth(lable))/2, (height+fm.getMaxAscent()-fm.getMaxDescent())/2);
		g.drawRect(0,0, width-1, height-1);
	}
}
/**
 * Close the connection
 */
private void quit ( ) throws IOException {
	//System.out.println("Quit");
	trans.doCommand("QUIT");
	trans.close();
	output("connection closed");
}
/**
 * Connects to the POP3-server and receives all messages. For each message a MessageEvent is generated.
 */
public void receiveMessages() {
	try {
		open();
		logon();
		int nr = getMessageList();
		for (int n=0; n<nr; n++) {
			message(getMessage(n));
			if (deleteMessages) deleteMessage(n);
		}
		quit();
	} catch (IOException ex) {
		output("Error in connection");
		//System.out.println("IOException: "+ex.getMessage());
	}
}
/**
 * The removePropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
	getPropertyChange().removePropertyChangeListener(listener);
}
/**
 * set-method for the deleteMessages property.
 * @param b boolean
 */
public void setDeleteMessages(boolean b) {
	deleteMessages = b;
}
/**
 * This method was created in VisualAge.
 * @param b boolean
 */
public void setInvisible(boolean b) {
	invisible = b;
}
/**
 * This method was created in VisualAge.
 * @param s java.lang.String
 */
public void setLable(String s) {
	lable = s;
}
/**
 * This method was created in VisualAge.
 * @param b boolean
 */
public void setMimeMessage(boolean b) {
	mimemessage = b;
}
/**
 * set-method for the password property.
 * @param password java.lang.String
 */
public void setPassword(String password) {
	this.password = password;
}
/**
 * set-method for the servername property.
 * @param servername java.lang.String
 */
public void setServername(String servername) {
	this.servername = servername;
}
/**
 * set-method for the username property.
 * @param name java.lang.String
 */
public void setUsername(String username) {
	this.username = username;
}
}