package mailbeans;

import java.net.*;
import java.io.*;
import java.util.*;
/**
 * This class deals with establishing a connection and the sending and receiving of data.
 * 
 */
public class Transport {
	Socket socket;
	InetAddress inetAddr;
	int port;
	
	protected BufferedReader inStream;
	protected DataOutputStream outStream;
	
/**
 * Creates a new Transport object and specifed the servername and the portnumber.
 * @param server java.lang.String
 * @param port int
 */
public Transport (String server, int port ) {
	try {
		inetAddr = InetAddress.getByName(server);
	} catch (UnknownHostException ex) {
		System.out.println("Unkown Host: "+ex.getMessage());
	}		
	this.port = port;	
}
/**
 * This method close the connection to the server.
 */
public void close ( ) throws IOException {
	inStream.close();
	outStream.close();
	socket.close();
}
/**
 * This method sends a command string to the server and returns the response.
 * @return java.lang.String
 * @param cmd java.lang.String
 * @exception java.io.IOException The exception description.
 */
public String doCommand(String cmd) throws java.io.IOException {
	outStream.writeBytes(cmd+"\n");
	return inStream.readLine();
}
/**
 * This method opens the connection to the server.
 * @return String
 */
public String open() throws IOException {
	socket = new Socket(inetAddr, port);

	inStream = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	outStream = new DataOutputStream(socket.getOutputStream());

	String response = inStream.readLine();

	return response;
}
/**
 * This method receives data from the server until the server sends a string that is equal with end parameter.
 * @param end java.lang.String
 * @exception java.io.IOException The exception description.
 */
public String[] receive(String end) throws IOException {
	Vector lines = new Vector();
	String line;
	String[] response;
	
	while ((line=inStream.readLine())!=null && !line.equals(end)) {
		lines.addElement(line);
	}
	if (line.equals(end)) {
		response = new String[lines.size()];
		lines.copyInto(response);
	} else throw new IOException("connection closed");
	return response;
}
/**
 * This method was created in VisualAge.
 * @param str java.lang.String
 */
public String send(String[] str, String end) throws IOException {
	for (int i=0; i<str.length; i++) {
		outStream.writeBytes(str[i]+"\n");
	}
	outStream.writeBytes(end+"\n");
	return inStream.readLine();
}
}